## Handling of moves
import pieces


type
    MoveFlag* = enum
        ## An enumeration of move flags
        Default = 0'u16,    # No flag
        EnPassant = 1,      # Move is a capture with en passant
        Capture = 2,        # Move is a capture
        DoublePush = 4,     # Move is a double pawn push
        # Castling metadata
        CastleLong = 8,
        CastleShort = 16,
        # Pawn promotion metadata
        PromoteToQueen = 32,
        PromoteToRook = 64,
        PromoteToBishop = 128,
        PromoteToKnight = 256    

    Move* = object
        ## A chess move
        startSquare*: Square
        targetSquare*: Square
        flags*: uint16

    MoveList* = object
        ## A list of moves
        data: array[218, Move]
        len: int8

func `[]`*(self: MoveList, i: SomeInteger): Move =
    when not defined(danger):
        if i >= self.len:
            raise newException(IndexDefect, &"move list access out of bounds ({i} >= {self.len})")
    result = self.data[i]


iterator items*(self: MoveList): Move =
    var i = 0
    while self.len > i:
        yield self.data[i]
        inc(i)


iterator pairs*(self: MoveList): tuple[i: int, move: Move] =
    var i = 0
    for item in self:
        yield (i, item)


func add*(self: var MoveList, move: Move) {.inline.} =
    self.data[self.len] = move
    inc(self.len)


func clear*(self: var MoveList) {.inline.} = 
    self.len = 0


func contains*(self: MoveList, move: Move): bool {.inline.} =
    for item in self:
        if move == item:
            return true
    return false


func len*(self: MoveList): int {.inline.} = self.len


# A bunch of move creation utilities

func createMove*(startSquare, targetSquare: Square, flags: varargs[MoveFlag]): Move =
    result = Move(startSquare: startSquare, targetSquare: targetSquare, flags: Default.uint16)
    for flag in flags:
        result.flags = result.flags or flag.uint16

proc createMove*(startSquare, targetSquare: string, flags: varargs[MoveFlag]): Move =
    result = createMove(startSquare.toSquare(), targetSquare.toSquare(), flags)

func createMove*(startSquare, targetSquare: SomeInteger, flags: varargs[MoveFlag]): Move =
    result = createMove(Square(startSquare.int8), Square(targetSquare.int8), flags)


func createMove*(startSquare: Square, targetSquare: SomeInteger, flags: varargs[MoveFlag]): Move =
    result = createMove(startSquare, Square(targetSquare.int8), flags)


func nullMove*: Move {.inline.} = createMove(nullSquare(), nullSquare())